#ifndef AudioProvider_h
#define AudioProvider_h

/*! \mainpage Partial Inheritance Use Case Example
 *
 *  This is an example of a use case where it would be useful for subclass's implementations
 *  of a virtual method to automatically inherit param-documentation from the superclass method;
 *  otherwise to avoid warnings (and missing documentation) for the subclasses, I have to 
 *  copy-and-paste a lot (since the parameter-names and their meanings are necessarily exactly
 *  the same for implementations of the same interface; if they weren't, it would break the object model)
 *  or I would have to leave the methods in the subclasses completely undocumented, in which case
 *  I am unable to communicate important information about their behavior to the documentation-reader.
 */

/** Abstract interface for any object that we can get a stream of fresh audio buffers from */
class IAudioProvider
{
public:
    /** This method should be implemented to return a reference to an AudioBuffer with the desired 
      * characteristics on success, or a NULL reference on failure.  The contents of the returned
      * AudioBuffer will be left up to the subclass.
      * @param numSamplesDesired how many samples' worth of audio we would like to get back
      * @param numChannelsDesired number of channels of audio desired (1==mono, 2==stereo, etc)
      * @param sampleRateHz the sample rate we intend to play back this audio at, in Hz (e.g. 44100 or 96000)
      * @returns a valid AudioBufferRef on success, or a NULL AudioBufferRef on failure
      */
    virtual AudioBufferRef GetNextAudioBuffer(int numSamplesDesired, int numChannelsDesired, int sampleRateHz) = 0;
};

/** This class returns audio from the specified .wav file */
class FileAudioProvider : public IAudioProvider
{
public:
    /** Constructor
      * @param filePath to a .wav file
      * @param loop true iff the audio should be played in an infinite loop; false to play it just once
      */
    FileAudioProvider(const char * filePath, bool loop);

    /** Implemented to return audio from the specified .wav file
      * @note if the file specified in the constructor wasn't found on the disk, we'll return NULL instead.
      * @note if we've reached the end of the file already, we'll return AudioBuffers full of silence,
      *       unless the "loop" parameter was set to true in the constructor, in which case we'll loop the audio.
      */
    virtual AudioBufferRef GetNextAudioBuffer(int numSamplesDesired, int numChannelsDesired, int sampleRateHz) = 0;

private:
    FILE * _wavFileHandle;
    bool _loop;
};

/** This class returns a square wave, for testing */
class SquareWaveAudioProvider : public IAudioProvider
{
public:
    /** Implemented to return audio that consists of 100 positive samples, followed by 100 negative
      * samples, and so on, indefinitely.
      * @note we try to return a reference to a static AudioBuffer where possible, for efficiency
      */
    virtual AudioBufferRef GetNextAudioBuffer(int numSamplesDesired, int numChannelsDesired, int sampleRateHz) = 0;
};

#endif
